/*
 * vim:tw=130:sw=4:ts=4
 *
 * Program name:	pure-config
 *
 * Description:		Parser-converter for main configuration pure-ftpd file
 *				 	written in C, on the top of libcfg+ - see:
 * 					http://platon.sk/projects/libcfg+/.
 *
 * Copyright:		Tomasz Wittner, 2004 under terms of GNU GPL v.2 license.
 *
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pure-config.h"

char *blank_char_pos(char *s);	/* returns pointer to first blank character (' ' or '\t') in s or NULL if any not found. */
char *rm_blanks(char *s);		/* removes blanks (' ' or '\t') from s . */
int usage(void);

int main(int argc, char **argv) {

	int i, j, ret;
	char *sc; /* place for semicolon instead blank between numbers */

	CFG_CONTEXT con = cfg_get_context(options);
	if(!con) {

		fprintf(stderr, "Error: can\'t allocate memory. Program exited.\n");

		if(opt[DEBUG])

			fprintf(stderr, "In %s at %d line program exited with %d code.\n", __FILE__, __LINE__ + 2, ENOMEM);

		exit(ENOMEM);
	}

	cfg_set_cmdline_context(con, 1, -1, argv);

	ret = cfg_parse(con);
	if(ret < 0) {

		cfg_fprint_error(con, stderr);
		fprintf(stderr, "\n");
	
		if(opt[DEBUG])

			fprintf(stderr, "In %s at %d line program exited with %d code.\n", __FILE__, __LINE__ + 2, ret);

		exit(ret);
	}
	
	if(opt[HELP])

		usage();
	
	ret = cfg_add_property(con, CFG_FILE_OPTION_ARG_SEPARATOR, " "); /* OK = ret 1, BAD = 0 */
	if(!ret) {

		cfg_fprint_error(con, stderr);
		fprintf(stderr, "\n");

		if(opt[DEBUG])

			fprintf(stderr, "In %s at %d line program exited with %d code.\n", __FILE__, __LINE__ + 2, ret);

        exit(ret);
    }

	ret = cfg_add_property(con, CFG_FILE_OPTION_ARG_SEPARATOR, "\t");
	if(!ret) {

		cfg_fprint_error(con, stderr);
		fprintf(stderr, "\n");

		if(opt[DEBUG])

			fprintf(stderr, "In %s at %d line program exited with %d code.\n", __FILE__, __LINE__ + 2, ret);

        exit(ret);
    }

	if(opt[CFGFILE]) {

		cfg_set_cfgfile_context(con, 0, -1, opt[CFGFILE]);
	}

	else {

		fprintf(stderr, "Error: file for parsing not given.\n");

		if(opt[DEBUG])

			fprintf(stderr, "In %s at %d line program exited with %d code.\n", __FILE__, __LINE__ + 2, ret);

		exit(1);
	}

	ret = cfg_parse(con);

	if(ret < 0) {

		cfg_fprint_error(con, stderr);
		fprintf(stderr, "\n");

		if(opt[DEBUG])
		
			fprintf(stderr, "In %s at %d line program exited with %d code.\n", __FILE__, __LINE__ + 2, ret);

		exit(ret);
	}

	for( i = 4 ; i <= OPTQNT - 1 ; i++) {

		if(opt[i] && strncasecmp("no", opt[i], 2)) {

			if(!strncasecmp("yes", opt[i], 3))

   				printf(" %s", output[i][(opt[LONGOPT]) ? 1 : 0]);

			else {

				if(	i >= SEMICOL_S && i <= SEMICOL_E && opt[i] ) { /* convert '1 2' to '1:2' */

					sc = opt[i];

					if( (sc = blank_char_pos(sc)) ) {
						
						*sc = ':';
						opt[i] = rm_blanks( opt[i] );
					}
					
				}

				printf(" %s%s", output[i][(opt[LONGOPT]) ? 1 : 0], opt[i]);
			}
		}
	}

	printf("\n");

	if(opt[DEBUG])
		
		fprintf(stderr, "In %s at %d line program exited with %d code.\n", __FILE__, __LINE__ + 2, EXIT_SUCCESS);

	return EXIT_SUCCESS;
}

char *rm_blanks(char *s) {

	char *p, *t;
   	 p = t = s;

	while( p && *p != '\0' ) {

		if( *p != ' ' && *p != '\t' ) {

			*t++ = *p++ ;
		}

		else

			p++ ;
	}

	*t = '\0' ;
	return s;
}

char *blank_char_pos(char *s) {

	char *t = s;

	for( ; t && *t != '\0' && *t != ' ' && *t != '\t' ; t++ )
		;

	if( t && *t != '\0' )

		return t;

	return NULL;
}

int usage(void) {
	
	puts("");
	puts("pure-config 	main pure-ftpd configuration file parser.");
	puts("		parses configuration file and prints pure-ftpd commandline"); 
	puts("		options on stdout.");
	puts("");
	puts("Usage:");
	puts("	pure-config -d, --debug				print some debug information on stderr");
	puts("	pure-config -f, --file=/path/to/pureftpd.conf	parse given pure-ftpd configuration file");
	puts("	pure-config -h, --help				print this message on stdout and exit");
	puts("	pure-config -l, --long				print long names of pure-ftpd options");
	puts("");
	puts("Example of usage:");
	puts("	pure-ftpd $(pure-config -f /etc/pure-ftpd/pureftpd.conf)");
	puts("");
	exit(EXIT_SUCCESS);
}
